import { VisualRegressionResult, TypeOption, DiffOption } from './plugin.js';
import 'pngjs';

declare global {
    namespace Cypress {
        interface Chainable {
            compareSnapshot(name: string, options?: PluginCommandOptions): Chainable<VisualRegressionResult>;
        }
    }
}
type ScreenshotOptions = Partial<Cypress.ScreenshotOptions & PluginSetupOptions>;
type PluginCommandOptions = number | ScreenshotOptions;
type PluginSetupOptions = {
    errorThreshold: number;
    failSilently: boolean;
};
type CypressConfigEnv = {
    visualRegressionType: TypeOption;
    visualRegressionBaseDirectory?: string;
    visualRegressionDiffDirectory?: string;
    visualRegressionGenerateDiff?: DiffOption;
    visualRegressionFailSilently?: boolean;
};
type TakeScreenshotProps = {
    path: string;
    size: number;
    dimensions: {
        width: number;
        height: number;
    };
    multipart: boolean;
    pixelRatio: number;
    takenAt: string;
    name: string;
    blackout: string[];
    duration: number;
    testAttemptIndex: number;
};
/** Add custom cypress command to compare image snapshots of an element or the window. */
declare function addCompareSnapshotCommand(screenshotOptions?: ScreenshotOptions): void;

export { type CypressConfigEnv, type PluginCommandOptions, type PluginSetupOptions, type ScreenshotOptions, type TakeScreenshotProps, addCompareSnapshotCommand };
